#include <Arduino.h>

const byte DATA_PIN = A3;
const byte CLOCK_PIN = A4;
const byte GAIN = 1;
bool pinsConfigured;

volatile float y1 = 20;                                             // 20 grams is calibration mass
volatile long x1; 
volatile long x0;
volatile float avg_size = 10.0; 

void HX711_init(uint8_t data_output_pin, uint8_t clock_pin) {       //set HX711 pins to MCU Registers, select gain
  uint8_t DATA_PIN = data_output_pin;
  uint8_t CLOCK_PIN = clock_pin;
  uint8_t GAIN = 1;
  bool pinsConfigured = true;
}

bool HX711_ReadytoSend() {
  if (!pinsConfigured) {                                            // Configure pins
    pinMode(DATA_PIN, INPUT);
    pinMode(CLOCK_PIN, OUTPUT);
    pinsConfigured = true;
  }
  return digitalRead(DATA_PIN) == LOW;                              // HX711 ready to send data when DATA_PIN is low
}

long Scale_read() {
  while (!HX711_ReadytoSend());                                      // Wait until ready to send data

  uint8_t data[3];                                                  // Create array of 4 bytes to hold data
  for (uint8_t j = 3; j--; ) {                                      // Shift data into array
    data[j] = shiftIn(DATA_PIN, CLOCK_PIN, MSBFIRST);
  }
  for (int i = 0; i < GAIN; i++) {                                  // set gain
    digitalWrite(CLOCK_PIN, HIGH);
    digitalWrite(CLOCK_PIN, LOW);
  }
  data[2] ^= 0x80;
  return ((uint32_t) data[2] << 16) | ((uint32_t) data[1] << 8) | (uint32_t) data[0];
}

void Scale_Tare(){
  HX711_init(DATA_PIN, CLOCK_PIN);                                  // init scale
  delay(500); 

  for(int i = 0; i< int(avg_size); i++){
    delay(10);
    x0 += Scale_read();
  }

  x0 /= long(avg_size);
  Serial.println("Add Calibration Mass");

  int i = 1;
  while(1){                                                         // Calibration
    if(Scale_read() < x0+1000){           
    } else {
      i++;
      delay(2000);
      for(int j = 0; j < int(avg_size); j++){
        x1 += Scale_read();
      }
      x1 /= long(avg_size);
      break;
    }
  }
  Serial.println("Calibration Complete"); 
}

void Scale_Output(){                                              // Call after tare to start outputting values
  long reading = 0; 
  for(int i = 0; i < int(avg_size); i++){
    reading += Scale_read();
  }
  reading /= long(avg_size);

  float ratio_1 = (float) (reading-x0);
  float ratio_2 = (float) (x1-x0);
  float ratio = ratio_1 / ratio_2; 
  float mass = y1*ratio;

  if(mass <= 0){
    mass = 0; 
  }
  Serial.print("Raw: ");
  Serial.print(reading);
  Serial.print(", ");
  Serial.println(mass);
}



int main() {
  init();
  Serial.begin(9600);

  Scale_Tare(); 

  while(1){
    Scale_Output();
    delay(50);
  }
}
